@@ -209,8 +209,7 @@ bool QCameraFormat::operator==(const QCameraFormat &other) const
     \snippet multimedia-snippets/camera.cpp Camera selection
 
     You can also use QCameraDevice to get general information about a camera
-    device such as description, physical position on the system, or camera sensor
-    orientation.
+    device such as description and physical position on the system.
 
     \snippet multimedia-snippets/camera.cpp Camera info
 
@@ -321,6 +320,38 @@ bool QCameraDevice::isDefault() const
     return d ? d->isDefault : false;
 }
 
+/*!
+    \since 6.7
+    \qmlproperty QtVideo::Rotation QtMultimedia::cameraDevice::correctionAngle
+
+    Returns the rotation angle needed to compensate for the physical camera rotation of the camera
+    compared to its native orientation. In other words, the property represents the clockwise angle
+    through which the output image needs to be rotated to be upright on the device screen in its
+    native orientation. Since \a correctionAngle is relative to the native orientation, this value
+    does not change with altering the device orientation (portrait/landscape). The correction angle
+    may be non-zero mostly on Android, where native and camera orientations are defined by the manufacturer.
+
+    \image camera_correctionAngle_90.png Example with 90 degrees \a correctionAngle
+*/
+
+/*!
+    \since 6.7
+    \property QCameraDevice::correctionAngle
+
+    Returns the rotation angle needed to compensate for the physical camera rotation of the camera
+    compared to its native orientation. In other words, the property represents the clockwise angle
+    through which the output image needs to be rotated to be upright on the device screen in its
+    native orientation. Since \a correctionAngle is relative to the native orientation, this value
+    does not change with altering the device orientation (portrait/landscape). The correction angle
+    may be non-zero mostly on Android, where native and camera orientations are defined by the manufacturer.
+
+    \image camera_correctionAngle_90.png Example with 90 degrees \a correctionAngle
+*/
+QtVideo::Rotation QCameraDevice::correctionAngle() const
+{
+    return d ? QtVideo::Rotation(d->orientation) : QtVideo::Rotation::None;
+}
+
 /*!
     \qmlproperty string QtMultimedia::cameraDevice::description
 
@@ -391,7 +422,7 @@ QList<QSize> QCameraDevice::photoResolutions() const
 }
 
 /*!
-    \qmlproperty CameraFormat QtMultiMedia::cameraDevice::videoFormats
+    \qmlproperty CameraFormat QtMultimedia::cameraDevice::videoFormats
 
     Holds the video formats supported by the camera.
 */
@@ -424,12 +455,16 @@ QCameraDevice& QCameraDevice::operator=(const QCameraDevice& other) = default;
 #ifndef QT_NO_DEBUG_STREAM
 QDebug operator<<(QDebug d, const QCameraDevice &camera)
 {
-    d.maybeSpace() << QStringLiteral("QCameraDevice(name=%1, position=%2, orientation=%3)")
-                          .arg(camera.description())
-                          .arg(QString::fromLatin1(QCamera::staticMetaObject.enumerator(QCamera::staticMetaObject.indexOfEnumerator("Position"))
-                               .valueToKey(camera.position())));
+    d.maybeSpace() << QStringLiteral("QCameraDevice(name=%1, id=%2, position=%3)")
+                              .arg(camera.description())
+                              .arg(QLatin1StringView(camera.id()))
+                              .arg(QLatin1StringView(
+                                      QMetaEnum::fromType<QCameraDevice::Position>().valueToKey(
+                                              camera.position())));
     return d.space();
 }
 #endif
 
 QT_END_NAMESPACE
+
+#include "moc_qcameradevice.cpp"
