@@ -133,7 +133,7 @@ void QAndroidCaptureSession::start(QMediaEncoderSettings &settings, const QUrl &
         return;
 
     if (!m_cameraSession && !m_audioInput) {
-        emit error(QMediaRecorder::ResourceError, QLatin1String("No devices are set"));
+        updateError(QMediaRecorder::ResourceError, QLatin1String("No devices are set"));
         return;
     }
 
@@ -142,13 +142,13 @@ void QAndroidCaptureSession::start(QMediaEncoderSettings &settings, const QUrl &
     const bool validCameraSession = m_cameraSession && m_cameraSession->camera();
 
     if (validCameraSession && !qt_androidCheckCameraPermission()) {
-        emit error(QMediaRecorder::ResourceError, QLatin1String("Camera permission denied."));
+        updateError(QMediaRecorder::ResourceError, QLatin1String("Camera permission denied."));
         setKeepAlive(false);
         return;
     }
 
     if (m_audioInput && !qt_androidCheckMicrophonePermission()) {
-        emit error(QMediaRecorder::ResourceError, QLatin1String("Microphone permission denied."));
+        updateError(QMediaRecorder::ResourceError, QLatin1String("Microphone permission denied."));
         setKeepAlive(false);
         return;
     }
@@ -171,7 +171,6 @@ void QAndroidCaptureSession::start(QMediaEncoderSettings &settings, const QUrl &
     }
 
     if (m_audioInput) {
-        m_mediaRecorder->setAudioSource(AndroidMediaRecorder::Camcorder);
         m_mediaRecorder->setAudioInput(m_audioInput->device.id());
         if (!m_mediaRecorder->isAudioSourceSet())
             m_mediaRecorder->setAudioSource(AndroidMediaRecorder::DefaultAudioSource);
@@ -217,36 +216,20 @@ void QAndroidCaptureSession::start(QMediaEncoderSettings &settings, const QUrl &
     m_outputLocationIsStandard = location.isEmpty() || QFileInfo(location).isRelative();
     m_mediaRecorder->setOutputFile(filePath);
 
-    // Even though the Android doc explicitly says that calling MediaRecorder.setPreviewDisplay()
-    // is not necessary when the Camera already has a Surface, it doesn't actually work on some
-    // devices. For example on the Samsung Galaxy Tab 2, the camera server dies after prepare()
-    // and start() if MediaRecorder.setPreviewDisplay() is not called.
     if (validCameraSession) {
-
-        if (m_cameraSession->videoOutput()) {
-            // When using a SurfaceTexture, we need to pass a new one to the MediaRecorder, not the
-            // same one that is set on the Camera or it will crash, hence the reset().
-            m_cameraSession->videoOutput()->reset();
-            if (m_cameraSession->videoOutput()->surfaceTexture())
-                m_mediaRecorder->setSurfaceTexture(
-                        m_cameraSession->videoOutput()->surfaceTexture());
-            else if (m_cameraSession->videoOutput()->surfaceHolder())
-                m_mediaRecorder->setSurfaceHolder(m_cameraSession->videoOutput()->surfaceHolder());
-        }
-
         m_cameraSession->disableRotation();
     }
 
     if (!m_mediaRecorder->prepare()) {
-        emit error(QMediaRecorder::FormatError, QLatin1String("Unable to prepare the media recorder."));
+        updateError(QMediaRecorder::FormatError,
+                    QLatin1String("Unable to prepare the media recorder."));
         restartViewfinder();
 
         return;
     }
 
     if (!m_mediaRecorder->start()) {
-        emit error(QMediaRecorder::FormatError,
-                   QMediaRecorderPrivate::msgFailedStartRecording());
+        updateError(QMediaRecorder::FormatError, QMediaRecorderPrivate::msgFailedStartRecording());
         restartViewfinder();
 
         return;
@@ -383,23 +366,7 @@ void QAndroidCaptureSession::restartViewfinder()
 
     if (m_cameraSession && m_cameraSession->camera()) {
         m_cameraSession->camera()->reconnect();
-
-        // This is not necessary on most devices, but it crashes on some if we don't stop the
-        // preview and reset the preview display on the camera when recording is over.
         m_cameraSession->camera()->stopPreviewSynchronous();
-
-        if (m_cameraSession->videoOutput()) {
-            // When using a SurfaceTexture, we need to pass a new one to the MediaRecorder, not the
-            // same one that is set on the Camera or it will crash, hence the reset().
-            m_cameraSession->videoOutput()->reset();
-            if (m_cameraSession->videoOutput()->surfaceTexture())
-                m_cameraSession->camera()->setPreviewTexture(
-                        m_cameraSession->videoOutput()->surfaceTexture());
-            else if (m_cameraSession->videoOutput()->surfaceHolder())
-                m_cameraSession->camera()->setPreviewDisplay(
-                        m_cameraSession->videoOutput()->surfaceHolder());
-        }
-
         m_cameraSession->camera()->startPreview();
         m_cameraSession->setReadyForCapture(true);
         m_cameraSession->enableRotation();
@@ -484,7 +451,7 @@ void QAndroidCaptureSession::onError(int what, int extra)
     Q_UNUSED(what);
     Q_UNUSED(extra);
     stop(true);
-    emit error(QMediaRecorder::ResourceError, QLatin1String("Unknown error."));
+    updateError(QMediaRecorder::ResourceError, QLatin1String("Unknown error."));
 }
 
 void QAndroidCaptureSession::onInfo(int what, int extra)
@@ -493,11 +460,11 @@ void QAndroidCaptureSession::onInfo(int what, int extra)
     if (what == 800) {
         // MEDIA_RECORDER_INFO_MAX_DURATION_REACHED
         stop();
-        emit error(QMediaRecorder::OutOfSpaceError, QLatin1String("Maximum duration reached."));
+        updateError(QMediaRecorder::OutOfSpaceError, QLatin1String("Maximum duration reached."));
     } else if (what == 801) {
         // MEDIA_RECORDER_INFO_MAX_FILESIZE_REACHED
         stop();
-        emit error(QMediaRecorder::OutOfSpaceError, QLatin1String("Maximum file size reached."));
+        updateError(QMediaRecorder::OutOfSpaceError, QLatin1String("Maximum file size reached."));
     }
 }
 
