@@ -3,34 +3,35 @@
 #ifndef QFFMPEG_P_H
 #define QFFMPEG_P_H
 
-#include <private/qtmultimediaglobal_p.h>
+//
+//  W A R N I N G
+//  -------------
+//
+// This file is not part of the Qt API. It exists purely as an
+// implementation detail. This header file may change from version to
+// version without notice, or even be removed.
+//
+// We mean it.
+//
+
+#include "qffmpegdefs_p.h"
+#include "qffmpegavaudioformat_p.h"
+
 #include <qstring.h>
 #include <optional>
 
-extern "C" {
-#include <libavformat/avformat.h>
-#include <libavcodec/avcodec.h>
-#include <libswresample/swresample.h>
-#include <libavutil/avutil.h>
-#include <libswscale/swscale.h>
+inline bool operator==(const AVRational &lhs, const AVRational &rhs)
+{
+    return lhs.num == rhs.num && lhs.den == rhs.den;
 }
 
-#define QT_FFMPEG_OLD_CHANNEL_LAYOUT (LIBAVCODEC_VERSION_INT < AV_VERSION_INT(59,24,100))
-#define QT_FFMPEG_HAS_VULKAN \
-  (LIBAVCODEC_VERSION_INT >= AV_VERSION_INT(58, 91, 100)) // since ffmpeg n4.3
-#define QT_FFMPEG_HAS_FRAME_TIME_BASE \
-  (LIBAVCODEC_VERSION_INT >= AV_VERSION_INT(59, 18, 100)) // since ffmpeg n5.0
-#define QT_FFMPEG_HAS_FRAME_DURATION \
-  (LIBAVCODEC_VERSION_INT >= AV_VERSION_INT(60, 3, 100)) // since ffmpeg n6.0
-#define QT_FFMPEG_STREAM_SIDE_DATA_DEPRECATED \
-  (LIBAVCODEC_VERSION_INT >= AV_VERSION_INT(60, 15, 100)) // since ffmpeg n6.1
-#define QT_FFMPEG_SWR_CONST_CH_LAYOUT \
-    (LIBSWRESAMPLE_VERSION_INT >= AV_VERSION_INT(4, 9, 100))
+inline bool operator!=(const AVRational &lhs, const AVRational &rhs)
+{
+    return !(lhs == rhs);
+}
 
 QT_BEGIN_NAMESPACE
 
-class QAudioFormat;
-
 namespace QFFmpeg
 {
 
@@ -102,6 +103,15 @@ struct AVDictionaryHolder
 
     operator AVDictionary **() { return &opts; }
 
+    AVDictionaryHolder() = default;
+
+    Q_DISABLE_COPY(AVDictionaryHolder)
+
+    AVDictionaryHolder(AVDictionaryHolder &&other) noexcept
+        : opts(std::exchange(other.opts, nullptr))
+    {
+    }
+
     ~AVDictionaryHolder()
     {
         if (opts)
@@ -178,22 +188,60 @@ Format findAVFormat(const Format *fmts, const Predicate &predicate)
     return findBestAVFormat(fmts, scoresGetter).first;
 }
 
-template<typename Format, typename CalculateScore>
-std::pair<Format, AVScore> findBestAVFormat(const Format *fmts,
-                                            const CalculateScore &calculateScore)
+template <typename Predicate>
+const AVCodecHWConfig *findHwConfig(const AVCodec *codec, const Predicate &predicate)
 {
-    std::pair result(Format(-1), NotSuitableAVScore);
-    if (fmts) {
-        for (; *fmts != -1 && result.second != BestAVScore; ++fmts) {
-            const auto score = calculateScore(*fmts);
+    for (int i = 0; const auto hwConfig = avcodec_get_hw_config(codec, i); ++i) {
+        if (predicate(hwConfig))
+            return hwConfig;
+    }
+
+    return nullptr;
+}
+
+template <typename Predicate>
+AVPixelFormat findAVPixelFormat(const AVCodec *codec, const Predicate &predicate)
+{
+    const AVPixelFormat format = findAVFormat(codec->pix_fmts, predicate);
+    if (format != AV_PIX_FMT_NONE)
+        return format;
+
+    auto checkHwConfig = [&predicate](const AVCodecHWConfig *config) {
+        return config->pix_fmt != AV_PIX_FMT_NONE && predicate(config->pix_fmt);
+    };
+
+    if (auto hwConfig = findHwConfig(codec, checkHwConfig))
+        return hwConfig->pix_fmt;
+
+    return AV_PIX_FMT_NONE;
+}
+
+template <typename Value, typename CalculateScore>
+auto findBestAVValue(const Value *values, const CalculateScore &calculateScore,
+                     Value invalidValue = {})
+{
+    using Limits = std::numeric_limits<decltype(calculateScore(*values))>;
+    std::pair result(invalidValue, Limits::min());
+    if (values) {
+        for (; *values != invalidValue && result.second != Limits::max(); ++values) {
+            const auto score = calculateScore(*values);
             if (score > result.second)
-                result = std::pair(*fmts, score);
+                result = { *values, score };
         }
     }
 
     return result;
 }
 
+template <typename Format, typename CalculateScore>
+std::pair<Format, AVScore> findBestAVFormat(const Format *fmts,
+                                            const CalculateScore &calculateScore)
+{
+    static_assert(std::is_same_v<Format, AVSampleFormat> || std::is_same_v<Format, AVPixelFormat>,
+                  "The input value is not AV format, use findBestAVValue instead.");
+    return findBestAVValue(fmts, calculateScore, Format(-1));
+}
+
 bool isHwPixelFormat(AVPixelFormat format);
 
 inline bool isSwPixelFormat(AVPixelFormat format)
@@ -201,27 +249,16 @@ inline bool isSwPixelFormat(AVPixelFormat format)
     return !isHwPixelFormat(format);
 }
 
+bool isAVCodecExperimental(const AVCodec *codec);
+
+void applyExperimentalCodecOptions(const AVCodec *codec, AVDictionary** opts);
+
 AVPixelFormat pixelFormatForHwDevice(AVHWDeviceType deviceType);
 
 const AVPacketSideData *streamSideData(const AVStream *stream, AVPacketSideDataType type);
 
-struct ResampleAudioFormat
-{
-    ResampleAudioFormat(const AVCodecParameters* codecPar);
-
-    ResampleAudioFormat(const QAudioFormat& audioFormat);
-
-#if QT_FFMPEG_OLD_CHANNEL_LAYOUT
-    uint64_t channelLayoutMask;
-#else
-    AVChannelLayout channelLayout;
-#endif
-    AVSampleFormat sampleFormat;
-    int sampleRate;
-};
-
-SwrContextUPtr createResampleContext(const ResampleAudioFormat& inputFormat,
-                                     const ResampleAudioFormat& outputFormat);
+SwrContextUPtr createResampleContext(const AVAudioFormat &inputFormat,
+                                     const AVAudioFormat &outputFormat);
 
 #ifdef Q_OS_DARWIN
 bool isCVFormatSupported(uint32_t format);
